@@ -1,6 +1,4 @@
 /*
- * $Id$
- *
  * Copyright (C) 2001-2003 FhG Fokus
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -15,20 +13,10 @@
  * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
-/*
- * 
- * History:
- * --------
- *  2004-02-20  removed from ser main.c into its own file (andrei)
- *  2004-03-04  moved setuid/setgid in do_suid() (andrei)
- *  2004-03-25  added increase_open_fds & set_core_dump (andrei)
- *  2004-05-03  applied pgid patch from janakj
- *  2007-06-07  added mlock_pages (no swap) support (andrei)
-  *             added set_rt_prio() (andrei)
- */
+
 /*!
  * \file
- * \brief SIP-router core :: 
+ * \brief Kamailio core :: Daemon init
  * \ingroup core
  * Module: \ref core
  */
@@ -500,7 +488,7 @@ int increase_open_fds(int target)
 		goto error;
 	}
 	orig=lim;
-	DBG("current open file limits: %lu/%lu\n",
+	LM_DBG("current open file limits: %lu/%lu\n",
 			(unsigned long)lim.rlim_cur, (unsigned long)lim.rlim_max);
 	if ((lim.rlim_cur==RLIM_INFINITY) || (target<=lim.rlim_cur))
 		/* nothing to do */
@@ -515,7 +503,7 @@ int increase_open_fds(int target)
 		lim.rlim_max=target;
 		lim.rlim_cur=target;
 	}
-	DBG("increasing open file limits to: %lu/%lu\n",
+	LM_DBG("increasing open file limits to: %lu/%lu\n",
 			(unsigned long)lim.rlim_cur, (unsigned long)lim.rlim_max);
 	if (setrlimit(RLIMIT_NOFILE, &lim)<0){
 		LM_CRIT("cannot increase the open file limit to"
@@ -592,7 +580,7 @@ int set_core_dump(int enable, long unsigned int size)
 		}
 	}
 done:
-	DBG("core dump limits set to %lu\n", (unsigned long)newlim.rlim_cur);
+	LM_DBG("core dump limits set to %lu\n", (unsigned long)newlim.rlim_cur);
 	return 0;
 error:
 	return -1;
