@@ -1,10 +1,9 @@
 /*
- * $Id$
- *
  * dispatcher module
  *
  * Copyright (C) 2004-2006 FhG Fokus
  * Copyright (C) 2005 Voice-System.ro
+ * Copyright (C) 2015 Daniel-Constantin Mierla (asipto.com)
  *
  * This file is part of Kamailio, a free SIP server.
  *
@@ -21,23 +20,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
- *
- * History
- * -------
- * 2004-07-31  first version, by daniel
- * 2005-04-22  added ruri  & to_uri hashing (andrei)
- * 2005-12-10  added failover support via avp (daniel)
- * 2006-08-15  added support for authorization username hashing (carsten)
- * 2007-01-11  Added a function to check if a specific gateway is in a
- * group (carsten)
- * 2007-01-12  Added a threshhold for automatic deactivation (carsten)
- * 2007-02-09  Added active probing of failed destinations and automatic
- * re-enabling of destinations (carsten)
- * 2007-05-08  Ported the changes to SVN-Trunk, renamed ds_is_domain to
- * ds_is_from_list and modified the function to work with IPv6 adresses.
- * 2007-07-18  removed index stuff
- * 			   added DB support to load/reload data(ancuta)
- * 2007-09-17  added list-file support for reload data (carstenbock)
  */
 
 /*! \file
@@ -232,6 +214,9 @@ int ds_set_attrs(ds_dest_t *dest, str *attrs)
 		} else if(pit->name.len==7
 				&& strncasecmp(pit->name.s, "maxload", 7)==0) {
 			str2sint(&pit->body, &dest->attrs.maxload);
+		} else if(pit->name.len==6
+				&& strncasecmp(pit->name.s, "socket", 6)==0) {
+			dest->attrs.socket = pit->body;
 		}
 	}
 	return 0;
@@ -253,6 +238,8 @@ int add_dest2list(int id, str uri, int flags, int priority, str *attrs,
 	struct hostent* he;
 	struct sip_uri puri;
 	int orig_id = 0, orig_nr = 0;
+	str host;
+	int port, proto;
 	ds_set_t *orig_ds_lists = ds_lists[list_idx];
 
 	/* check uri */
@@ -315,6 +302,7 @@ int add_dest2list(int id, str uri, int flags, int priority, str *attrs,
 	strncpy(dp->uri.s, uri.s, uri.len);
 	dp->uri.s[uri.len]='\0';
 	dp->uri.len = uri.len;
+
 	dp->flags = flags;
 	dp->priority = priority;
 
@@ -324,6 +312,22 @@ int add_dest2list(int id, str uri, int flags, int priority, str *attrs,
 		goto err;
 	}
 
+	/* check socket attribute */
+	if (dp->attrs.socket.s && dp->attrs.socket.len > 0) {
+		if (parse_phostport(dp->attrs.socket.s, &host.s, &host.len,
+				&port, &proto)!=0) {
+			LM_ERR("bad socket <%.*s>\n", dp->attrs.socket.len, dp->attrs.socket.s);
+			goto err;
+		}
+		dp->sock = grep_sock_info( &host, (unsigned short)port, proto);
+		if (dp->sock==0) {
+			LM_ERR("non-local socket <%.*s>\n", dp->attrs.socket.len, dp->attrs.socket.s);
+			goto err;
+		}
+	} else if (ds_default_sockinfo) {
+		dp->sock = ds_default_sockinfo;
+	}
+
 	/* The Hostname needs to be \0 terminated for resolvehost, so we
 	 * make a copy here. */
 	strncpy(hn, puri.host.s, puri.host.len);
@@ -340,7 +344,7 @@ int add_dest2list(int id, str uri, int flags, int priority, str *attrs,
 	hostent2ip_addr(&dp->ip_address, he, 0);
 
 	/* Copy the port out of the URI */
-	dp->port = puri.port_no;		
+	dp->port = puri.port_no;
 	/* Copy the proto out of the URI */
 	dp->proto = puri.proto;
 
@@ -593,11 +597,11 @@ int ds_load_list(char *lfile)
 		while(*p && (*p==' ' || *p=='\t' || *p=='\r' || *p=='\n'))
 			p++;
 		if(*p=='\0' || *p=='#')
-			goto add_destination; /* no priority given */
+			goto add_destination; /* no attrs given */
 
 		/* get attributes */
 		attrs.s = p;
-		while(*p && *p!=' ' && *p!='\t' && *p!='\r' && *p!='\n' && *p!='#')
+		while(*p && *p!=' ' && *p!='\t' && *p!='\r' && *p!='\n')
 			p++;
 		attrs.len = p-attrs.s;
 
@@ -1266,6 +1270,33 @@ static inline int ds_get_index(int group, ds_set_t **index)
 	return 0;
 }
 
+/*
+ * Check if a destination set exists
+ */
+int ds_list_exist(int set)
+{
+	ds_set_t *si = NULL;
+	LM_DBG("-- Looking for set %d\n", set);
+
+	/* get the index of the set */
+	si = _ds_list;
+	while(si)
+	{
+		if(si->id == set)
+		{
+			break;
+		}
+		si = si->next;
+	}
+
+	if(si==NULL)
+	{
+		LM_INFO("destination set [%d] not found\n", set);
+		return -1;	/* False */
+	}
+	LM_INFO("destination set [%d] found\n", set);
+	return 1;	/* True */
+}
 
 /**
  *
@@ -1570,7 +1601,8 @@ int ds_load_unset(struct sip_msg *msg)
 /**
  *
  */
-static inline int ds_update_dst(struct sip_msg *msg, str *uri, int mode)
+static inline int ds_update_dst(struct sip_msg *msg, str *uri,
+								struct socket_info *sock, int mode)
 {
 	struct action act;
 	struct run_act_ctx ra_ctx;
@@ -1601,6 +1633,8 @@ static inline int ds_update_dst(struct sip_msg *msg, str *uri, int mode)
 			ruri_mark_new(); /* re-use uri for serial forking */
 			break;
 	}
+	if (sock)
+		msg->force_send_socket = sock;
 	return 0;
 }
 
@@ -1625,6 +1659,7 @@ int ds_select_dst_limit(sip_msg_t *msg, int set, int alg, unsigned int limit, in
 	unsigned int hash;
 	int_str avp_val;
 	ds_set_t *idx = NULL;
+	char buf[2+16+1];
 
 	if(msg==NULL)
 	{
@@ -1802,7 +1837,7 @@ int ds_select_dst_limit(sip_msg_t *msg, int set, int alg, unsigned int limit, in
 
 	hash = i;
 
-	if(ds_update_dst(msg, &idx->dlist[hash].uri, mode)!=0)
+	if(ds_update_dst(msg, &idx->dlist[hash].uri, idx->dlist[hash].sock, mode)!=0)
 	{
 		LM_ERR("cannot set dst addr\n");
 		return -1;
@@ -1833,6 +1868,17 @@ int ds_select_dst_limit(sip_msg_t *msg, int set, int alg, unsigned int limit, in
 							avp_val)!=0)
 					return -1;
 			}
+
+			/* only add sock_avp if dst_avp is set */
+			if(sock_avp_name.n!=0 && idx->dlist[idx->nr-1].sock)
+			{
+				avp_val.s.len = 1 + sprintf(buf, "%p", idx->dlist[idx->nr-1].sock);
+				avp_val.s.s = buf;
+				if(add_avp(AVP_VAL_STR|sock_avp_type, sock_avp_name,
+							avp_val)!=0)
+					return -1;
+			}
+
 			if(alg==DS_ALG_LOAD)
 			{
 				if(idx->dlist[idx->nr-1].attrs.duid.len<=0)
@@ -1853,7 +1899,7 @@ int ds_select_dst_limit(sip_msg_t *msg, int set, int alg, unsigned int limit, in
 		/* add to avp */
 
 		for(i=hash-1; i>=0 && cnt<limit; i--)
-		{	
+		{
 			if(ds_skip_dst(idx->dlist[i].flags)
 					|| (ds_use_default!=0 && i==(idx->nr-1)))
 				continue;
@@ -1873,6 +1919,16 @@ int ds_select_dst_limit(sip_msg_t *msg, int set, int alg, unsigned int limit, in
 							avp_val)!=0)
 					return -1;
 			}
+
+			if(sock_avp_name.n!=0 && idx->dlist[i].sock)
+			{
+				avp_val.s.len = 1 + sprintf(buf, "%p", idx->dlist[i].sock);
+				avp_val.s.s = buf;
+				if(add_avp(AVP_VAL_STR|sock_avp_type, sock_avp_name,
+							avp_val)!=0)
+					return -1;
+			}
+
 			if(alg==DS_ALG_LOAD)
 			{
 				if(idx->dlist[i].attrs.duid.len<=0)
@@ -1891,7 +1947,7 @@ int ds_select_dst_limit(sip_msg_t *msg, int set, int alg, unsigned int limit, in
 		}
 
 		for(i=idx->nr-1; i>hash && cnt<limit; i--)
-		{	
+		{
 			if(ds_skip_dst(idx->dlist[i].flags)
 					|| (ds_use_default!=0 && i==(idx->nr-1)))
 				continue;
@@ -1910,6 +1966,16 @@ int ds_select_dst_limit(sip_msg_t *msg, int set, int alg, unsigned int limit, in
 							avp_val)!=0)
 					return -1;
 			}
+
+			if(sock_avp_name.n!=0 && idx->dlist[i].sock)
+			{
+				avp_val.s.len = 1 + sprintf(buf, "%p", idx->dlist[i].sock);
+				avp_val.s.s = buf;
+				if(add_avp(AVP_VAL_STR|sock_avp_type, sock_avp_name,
+							avp_val)!=0)
+					return -1;
+			}
+
 			if(alg==DS_ALG_LOAD)
 			{
 				if(idx->dlist[i].attrs.duid.len<=0)
@@ -1939,6 +2005,15 @@ int ds_select_dst_limit(sip_msg_t *msg, int set, int alg, unsigned int limit, in
 						avp_val)!=0)
 				return -1;
 		}
+		if(sock_avp_name.n!=0 && idx->dlist[hash].sock)
+		{
+			avp_val.s.len = 1 + sprintf(buf, "%p", idx->dlist[hash].sock);
+			avp_val.s.s = buf;
+			if(add_avp(AVP_VAL_STR|sock_avp_type, sock_avp_name,
+						avp_val)!=0)
+				return -1;
+		}
+
 		if(alg==DS_ALG_LOAD)
 		{
 			if(idx->dlist[hash].attrs.duid.len<=0)
@@ -1980,7 +2055,9 @@ int ds_next_dst(struct sip_msg *msg, int mode)
 	struct search_state st;
 	struct usr_avp *avp;
 	struct usr_avp *prev_avp;
+	struct socket_info *sock = NULL;
 	int_str avp_value;
+	int_str sock_avp_value;
 	int alg = 0;
 
 	if(!(ds_flags&DS_FAILOVER_ON) || dst_avp_name.n==0)
@@ -2012,6 +2089,18 @@ int ds_next_dst(struct sip_msg *msg, int mode)
 		}
 	}
 
+	if(sock_avp_name.n!=0)
+	{
+		prev_avp = search_first_avp(sock_avp_type,
+				attrs_avp_name, &sock_avp_value, &st);
+		if(prev_avp!=NULL)
+		{
+			if (sscanf( sock_avp_value.s.s, "%p", (void**)&sock ) != 1)
+				sock = NULL;
+			destroy_avp(prev_avp);
+		}
+	}
+
 	prev_avp = search_first_avp(dst_avp_type, dst_avp_name, &avp_value, &st);
 	if(prev_avp==NULL)
 		return -1; /* used avp deleted -- strange */
@@ -2021,7 +2110,7 @@ int ds_next_dst(struct sip_msg *msg, int mode)
 	if(avp==NULL || !(avp->flags&AVP_VAL_STR))
 		return -1; /* no more avps or value is int */
 
-	if(ds_update_dst(msg, &avp_value.s, mode)!=0)
+	if(ds_update_dst(msg, &avp_value.s, sock, mode)!=0)
 	{
 		LM_ERR("cannot set dst addr\n");
 		return -1;
@@ -2078,12 +2167,49 @@ int ds_mark_dst(struct sip_msg *msg, int state)
 }
 
 /**
- *
+ * Get state for given destination
+ */
+int ds_get_state(int group, str *address)
+{
+	int i=0;
+	int state = 0;
+	ds_set_t *idx = NULL;
+
+	if(_ds_list==NULL || _ds_list_nr<=0)
+	{
+		LM_ERR("the list is null\n");
+		return -1;
+	}
+
+	/* get the index of the set */
+	if(ds_get_index(group, &idx)!=0)
+	{
+		LM_ERR("destination set [%d] not found\n", group);
+		return -1;
+	}
+
+	while(i<idx->nr)
+	{
+		if(idx->dlist[i].uri.len==address->len
+				&& strncasecmp(idx->dlist[i].uri.s, address->s,
+					address->len)==0)
+		{
+			/* destination address found */
+			state = idx->dlist[i].flags;
+		}
+		i++;
+	}
+	return state;
+}
+
+/**
+ * Update destionation's state
  */
 int ds_update_state(sip_msg_t *msg, int group, str *address, int state)
 {
 	int i=0;
 	int old_state = 0;
+	int init_state = 0;
 	ds_set_t *idx = NULL;
 
 	if(_ds_list==NULL || _ds_list_nr<=0)
@@ -2111,6 +2237,9 @@ int ds_update_state(sip_msg_t *msg, int group, str *address, int state)
 			/* reset the bits used for states */
 			idx->dlist[i].flags &= ~(DS_STATES_ALL);
 
+			/* we need the initial state for inactive counter */
+			init_state = state;
+
 			if((state & DS_TRYING_DST) && (old_state & DS_INACTIVE_DST))
 			{
 				/* old state is inactive, new state is trying => keep it inactive
@@ -2129,15 +2258,30 @@ int ds_update_state(sip_msg_t *msg, int group, str *address, int state)
 
 			if(state & DS_TRYING_DST)
 			{
-				idx->dlist[i].failure_count++;
-				if (idx->dlist[i].failure_count >= probing_threshhold)
+				idx->dlist[i].message_count++;
+				/* Destination is not replying.. Increasing failure counter */
+				if (idx->dlist[i].message_count >= probing_threshold)
 				{
+					/* Destionation has too much lost messages.. Bringing it to inactive state */
 					idx->dlist[i].flags &= ~DS_TRYING_DST;
 					idx->dlist[i].flags |= DS_INACTIVE_DST;
-					idx->dlist[i].failure_count = 0;
+					idx->dlist[i].message_count = 0;
 				}
 			} else {
-				idx->dlist[i].failure_count = 0;
+				if(!(init_state & DS_TRYING_DST) && (old_state & DS_INACTIVE_DST)){
+					idx->dlist[i].message_count++;
+					/* Destination was inactive but it is just replying.. Increasing successful counter */
+					if (idx->dlist[i].message_count < inactive_threshold)
+					{
+						/* Destination has not enough successful replies.. Leaving it into inactive state */
+						idx->dlist[i].flags |= DS_INACTIVE_DST;
+					}else{
+						/* Destination has enough replied messages.. Bringing it to active state */
+						idx->dlist[i].message_count = 0;
+					}
+				}else{
+					idx->dlist[i].message_count = 0;
+				}
 			}
 
 			if (!ds_skip_dst(old_state) && ds_skip_dst(idx->dlist[i].flags))
@@ -2266,10 +2410,10 @@ int ds_print_list(FILE *fout)
 			else if (list->dlist[j].flags&DS_TRYING_DST) {
 				fprintf(fout, "    Trying");
 				/* print the tries for this host. */
-				if (list->dlist[j].failure_count > 0) {
+				if (list->dlist[j].message_count > 0) {
 					fprintf(fout, " (Fail %d/%d)",
-							list->dlist[j].failure_count,
-							probing_threshhold);
+							list->dlist[j].message_count,
+							probing_threshold);
 				} else {
 					fprintf(fout, "           ");
 				}
@@ -2495,7 +2639,8 @@ static void ds_options_callback( struct cell *t, int type,
 		state = 0;
 		if (ds_probing_mode==DS_PROBE_ALL)
 			state |= DS_PROBING_DST;
-		if (ds_update_state(fmsg, group, &uri, state) != 0)
+		/* Check if in the meantime someone disabled the target through RPC or MI */
+		if (!(ds_get_state(group, &uri) & DS_DISABLED_DST) && ds_update_state(fmsg, group, &uri, state) != 0)
 		{
 			LM_ERR("Setting the state failed (%.*s, group %d)\n", uri.len,
 					uri.s, group);
@@ -2504,8 +2649,8 @@ static void ds_options_callback( struct cell *t, int type,
 		state = DS_TRYING_DST;
 		if (ds_probing_mode!=DS_PROBE_NONE)
 			state |= DS_PROBING_DST;
-
-		if (ds_update_state(fmsg, group, &uri, state) != 0)
+		/* Check if in the meantime someone disabled the target through RPC or MI */
+		if (!(ds_get_state(group, &uri) & DS_DISABLED_DST) && ds_update_state(fmsg, group, &uri, state) != 0)
 		{
 			LM_ERR("Setting the probing state failed (%.*s, group %d)\n",
 					uri.len, uri.s, group);
@@ -2555,6 +2700,11 @@ void ds_check_timer(unsigned int ticks, void* param)
 				set_uac_req(&uac_r, &ds_ping_method, 0, 0, 0,
 						TMCB_LOCAL_COMPLETED, ds_options_callback,
 						(void*)(long)list->id);
+				if (list->dlist[j].attrs.socket.s != NULL && list->dlist[j].attrs.socket.len > 0) {
+					uac_r.ssock = &list->dlist[j].attrs.socket;
+				} else if (ds_default_socket.s != NULL && ds_default_socket.len > 0) {
+					uac_r.ssock = &ds_default_socket;
+				}
 				if (tmb.t_request(&uac_r,
 							&list->dlist[j].uri,
 							&list->dlist[j].uri,
