@@ -1,6 +1,4 @@
 /*
- * $Id$
- *
  * Process REGISTER request and send reply
  *
  * Copyright (C) 2001-2003 FhG Fokus
@@ -22,21 +20,6 @@
  * along with this program; if not, write to the Free Software 
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- * History:
- * ----------
- * 2003-01-27 next baby-step to removing ZT - PRESERVE_ZT (jiri)
- * 2003-02-28 scrathcpad compatibility abandoned (jiri)
- * 2003-03-21 save_noreply added, patch provided by Maxim Sobolev 
- *            <sobomax@portaone.com> (janakj)
- * 2005-07-11 added sip_natping_flag for nat pinging with SIP method
- *            instead of UDP package (bogdan)
- * 2006-04-13 added tcp_persistent_flag for keeping the TCP connection as long
- *            as a TCP contact is registered (bogdan)
- * 2006-11-22 save_noreply and save_memory merged into save() (bogdan)
- * 2006-11-28 Added statistic support for the number of accepted/rejected 
- *            registrations. (Jeffrey Magder - SOMA Networks) 
- * 2007-02-24  sip_natping_flag moved into branch flags, so migrated to 
- *             nathelper module (bogdan)
  */
 /*!
  * \file
@@ -234,7 +217,8 @@ static inline int no_contacts(sip_msg_t *_m, udomain_t* _d, str* _a, str* _h)
 /*! \brief
  * Fills the common part (for all contacts) of the info structure
  */
-static inline ucontact_info_t* pack_ci( struct sip_msg* _m, contact_t* _c, unsigned int _e, unsigned int _f, int _use_regid)
+static inline ucontact_info_t* pack_ci( struct sip_msg* _m, contact_t* _c,
+		unsigned int _e, unsigned int _f, int _use_regid)
 {
 	static ucontact_info_t ci;
 	static str no_ua = str_init("n/a");
@@ -322,6 +306,8 @@ static inline ucontact_info_t* pack_ci( struct sip_msg* _m, contact_t* _c, unsig
 			ci.received = path_received;
 		}
 
+		ci.server_id = server_id;
+
 		allow_parsed = 0; /* not parsed yet */
 		received_found = 0; /* not found yet */
 		m = _m; /* remember the message */
