@@ -1,6 +1,6 @@
-/* $Id$
- *
+/*
  * Copyright (C) 2003-2008 Sippy Software, Inc., http://www.sippysoft.com
+ * Copyright (C) 2014-2015 Sipwise GmbH, http://www.sipwise.com
  *
  * This file is part of Kamailio, a free SIP server.
  *
@@ -17,152 +17,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
- *
- * History:
- * ---------
- * 2003-10-09	nat_uac_test introduced (jiri)
- *
- * 2003-11-06   nat_uac_test permitted from onreply_route (jiri)
- *
- * 2003-12-01   unforce_rtp_proxy introduced (sobomax)
- *
- * 2004-01-07	RTP proxy support updated to support new version of the
- *		RTP proxy (20040107).
- *
- *		force_rtp_proxy() now inserts a special flag
- *		into the SDP body to indicate that this session already
- *		proxied and ignores sessions with such flag.
- *
- *		Added run-time check for version of command protocol
- *		supported by the RTP proxy.
- *
- * 2004-01-16   Integrated slightly modified patch from Tristan Colgate,
- *		force_rtp_proxy function with IP as a parameter (janakj)
- *
- * 2004-01-28	nat_uac_test extended to allow testing SDP body (sobomax)
- *
- *		nat_uac_test extended to allow testing top Via (sobomax)
- *
- * 2004-02-21	force_rtp_proxy now accepts option argument, which
- *		consists of string of chars, each of them turns "on"
- *		some feature, currently supported ones are:
- *
- *		 `a' - flags that UA from which message is received
- *		       doesn't support symmetric RTP;
- *		 `l' - force "lookup", that is, only rewrite SDP when
- *		       corresponding session is already exists in the
- *		       RTP proxy. Only makes sense for SIP requests,
- *		       replies are always processed in "lookup" mode;
- *		 `i' - flags that message is received from UA in the
- *		       LAN. Only makes sense when RTP proxy is running
- *		       in the bridge mode.
- *
- *		force_rtp_proxy can now be invoked without any arguments,
- *		as previously, with one argument - in this case argument
- *		is treated as option string and with two arguments, in
- *		which case 1st argument is option string and the 2nd
- *		one is IP address which have to be inserted into
- *		SDP (IP address on which RTP proxy listens).
- *
- * 2004-03-12	Added support for IPv6 addresses in SDPs. Particularly,
- *		force_rtp_proxy now can work with IPv6-aware RTP proxy,
- *		replacing IPv4 address in SDP with IPv6 one and vice versa.
- *		This allows creating full-fledged IPv4<->IPv6 gateway.
- *		See 4to6.cfg file for example.
- *
- *		Two new options added into force_rtp_proxy:
- *
- *		 `f' - instructs nathelper to ignore marks inserted
- *		       by another nathelper in transit to indicate
- *		       that the session is already goes through another
- *		       proxy. Allows creating chain of proxies.
- *		 `r' - flags that IP address in SDP should be trusted.
- *		       Without this flag, nathelper ignores address in the
- *		       SDP and uses source address of the SIP message
- *		       as media address which is passed to the RTP proxy.
- *
- *		Protocol between nathelper and RTP proxy in bridge
- *		mode has been slightly changed. Now RTP proxy expects SER
- *		to provide 2 flags when creating or updating session
- *		to indicate direction of this session. Each of those
- *		flags can be either `e' or `i'. For example `ei' means
- *		that we received INVITE from UA on the "external" network
- *		network and will send it to the UA on "internal" one.
- *		Also possible `ie' (internal->external), `ii'
- *		(internal->internal) and `ee' (external->external). See
- *		example file alg.cfg for details.
- *
- * 2004-03-15	If the rtp proxy test failed (wrong version or not started)
- *		retry test from time to time, when some *rtpproxy* function
- *		is invoked. Minimum interval between retries can be
- *		configured via rtpproxy_disable_tout module parameter (default
- *		is 60 seconds). Setting it to -1 will disable periodic
- *		rechecks completely, setting it to 0 will force checks
- *		for each *rtpproxy* function call. (andrei)
- *
- * 2004-03-22	Fix assignment of rtpproxy_retr and rtpproxy_tout module
- *		parameters.
- *
- * 2004-03-22	Fix get_body position (should be called before get_callid)
- * 				(andrei)
- *
- * 2004-03-24	Fix newport for null ip address case (e.g onhold re-INVITE)
- * 				(andrei)
- *
- * 2004-09-30	added received port != via port test (andrei)
- *
- * 2004-10-10   force_socket option introduced (jiri)
- *
- * 2005-02-24	Added support for using more than one rtp proxy, in which
- *		case traffic will be distributed evenly among them. In addition,
- *		each such proxy can be assigned a weight, which will specify
- *		which share of the traffic should be placed to this particular
- *		proxy.
- *
- *		Introduce failover mechanism, so that if SER detects that one
- *		of many proxies is no longer available it temporarily decreases
- *		its weight to 0, so that no traffic will be assigned to it.
- *		Such "disabled" proxies are periodically checked to see if they
- *		are back to normal in which case respective weight is restored
- *		resulting in traffic being sent to that proxy again.
- *
- *		Those features can be enabled by specifying more than one "URI"
- *		in the rtpproxy_sock parameter, optionally followed by the weight,
- *		which if absent is assumed to be 1, for example:
- *
- *		rtpproxy_sock="unix:/foo/bar=4 udp:1.2.3.4:3456=3 udp:5.6.7.8:5432=1"
- *
- * 2005-02-25	Force for pinging the socket returned by USRLOC (bogdan)
- *
- * 2005-03-22	support for multiple media streams added (netch)
- *
- * 2005-07-11  SIP ping support added (bogdan)
- *
- * 2005-07-14  SDP origin (o=) IP may be also changed (bogdan)
- *
- * 2006-03-08  fix_nated_sdp() may take one more param to force a specific IP;
- *             force_rtp_proxy() accepts a new flag 's' to swap creation/
- *              confirmation between requests/replies;
- *             add_rcv_param() may take as parameter a flag telling if the
- *              parameter should go to the contact URI or contact header;
- *             (bogdan)
- * 2006-03-28 Support for changing session-level SDP connection (c=) IP when
- *            media-description also includes connection information (bayan)
- * 2007-04-13 Support multiple sets of rtpproxies and set selection added
- *            (ancuta)
- * 2007-04-26 Added some MI commands:
- *             nh_enable_ping used to enable or disable natping
- *             nh_enable_rtpp used to enable or disable a specific rtp proxy
- *             nh_show_rtpp   used to display information for all rtp proxies
- *             (ancuta)
- * 2007-05-09 New function start_recording() allowing to start recording RTP
- *             session in the RTP proxy (Carsten Bock - ported from SER)
- * 2007-09-11 Separate timer process and support for multiple timer processes
- *             (bogdan)
- * 2008-12-12 Support for RTCP attribute in the SDP
- *              (Min Wang/BASIS AudioNet - ported from SER)
- * 2010-08-05 Core SDP parser integrated into nathelper (osas)
- * 2010-10-08 Removal of deprecated force_rtp_proxy and swap flag (osas)
  */
 
 #include <sys/types.h>
@@ -189,6 +43,9 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
+#include <ifaddrs.h>
+#include <netinet/in.h>
+#include <arpa/inet.h>
 
 #include "../../flags.h"
 #include "../../sr_module.h"
@@ -241,7 +98,7 @@ MODULE_VERSION
 
 
 #define DEFAULT_RTPP_SET_ID		0
-
+#define MAX_RTPP_TRIED_NODES    50
 #define MI_SET_NATPING_STATE		"nh_enable_ping"
 #define MI_DEFAULT_NATPING_STATE	1
 
@@ -255,6 +112,8 @@ MODULE_VERSION
 #define MI_RTP_PROXY_NOT_FOUND_LEN	(sizeof(MI_RTP_PROXY_NOT_FOUND)-1)
 #define MI_PING_DISABLED			"NATping disabled from script"
 #define MI_PING_DISABLED_LEN		(sizeof(MI_PING_DISABLED)-1)
+#define MI_DISABLED_PERMANENT        "1 (permanent)"
+#define MI_DISABLED_PERMANENT_LEN    (sizeof(MI_DISABLED_PERMANENT)-1)
 #define MI_SET						"set"
 #define MI_SET_LEN					(sizeof(MI_SET)-1)
 #define MI_INDEX					"index"
@@ -316,6 +175,10 @@ static int mod_init(void);
 static int child_init(int);
 static void mod_destroy(void);
 
+static int get_ip_type(char *str_addr);
+static int get_ip_scope(char *str_addr); // useful for link-local ipv6
+static int bind_force_send_ip(int sock_idx);
+
 /* Pseudo-Variables */
 static int pv_get_rtpstat_f(struct sip_msg *, pv_param_t *, pv_value_t *);
 
@@ -328,7 +191,8 @@ static struct mi_root* mi_show_rtpproxies(struct mi_root* cmd_tree,
 
 static int rtpengine_disable_tout = 60;
 static int rtpengine_retr = 5;
-static int rtpengine_tout = 1;
+static int rtpengine_tout_ms = 1000;
+static int queried_nodes_limit = MAX_RTPP_TRIED_NODES;
 static pid_t mypid;
 static unsigned int myseqn = 0;
 static str extra_id_pv_param = {NULL, 0};
@@ -347,6 +211,9 @@ static struct rtpp_set * default_rtpp_set=0;
 
 static str body_intermediate;
 
+static str rtp_inst_pv_param = {NULL, 0};
+static pv_spec_t *rtp_inst_pvar = NULL;
+
 /* array with the sockets used by rtpporxy (per process)*/
 static unsigned int rtpp_no = 0;
 static int *rtpp_socks = 0;
@@ -354,6 +221,13 @@ static int *rtpp_socks = 0;
 static int     setid_avp_type;
 static int_str setid_avp;
 
+static str            write_sdp_pvar_str = {NULL, 0};
+static pv_spec_t*     write_sdp_pvar = NULL;
+
+
+char* force_send_ip_str="";
+int force_send_ip_af = AF_UNSPEC;
+
 typedef struct rtpp_set_link {
 	struct rtpp_set *rset;
 	pv_spec_t *rpv;
@@ -415,12 +289,16 @@ static param_export_t params[] = {
 	                         (void*)rtpengine_set_store          },
 	{"rtpengine_disable_tout",INT_PARAM, &rtpengine_disable_tout },
 	{"rtpengine_retr",        INT_PARAM, &rtpengine_retr         },
-	{"rtpengine_tout",        INT_PARAM, &rtpengine_tout         },
+	{"rtpengine_tout_ms",     INT_PARAM, &rtpengine_tout_ms      },
+	{"queried_nodes_limit",   INT_PARAM, &queried_nodes_limit    },
 	{"db_url",                PARAM_STR, &rtpp_db_url },
 	{"table_name",            PARAM_STR, &rtpp_table_name },
 	{"url_col",               PARAM_STR, &rtpp_url_col },
 	{"extra_id_pv",           PARAM_STR, &extra_id_pv_param },
 	{"setid_avp",             PARAM_STRING, &setid_avp_param },
+	{"force_send_interface",  PARAM_STRING, &force_send_ip_str	},
+	{"rtp_inst_pvar",         PARAM_STR, &rtp_inst_pv_param },
+	{"write_sdp_pv",          PARAM_STR, &write_sdp_pvar_str          },
 	{0, 0, 0}
 };
 
@@ -447,6 +325,140 @@ struct module_exports exports = {
 };
 
 
+static int get_ip_type(char *str_addr)
+{
+	struct addrinfo hint, *info = NULL;
+	int ret;
+
+	memset(&hint, '\0', sizeof hint);
+	hint.ai_family = PF_UNSPEC;
+	hint.ai_flags = AI_NUMERICHOST;
+
+	ret = getaddrinfo(str_addr, NULL, &hint, &info);
+	if (ret) {
+		/* Invalid ip addinfos */
+		return -1;
+	}
+
+	if(info->ai_family == AF_INET) {
+		LM_DBG("%s is an ipv4 addinfos\n", str_addr);
+	} else if (info->ai_family == AF_INET6) {
+		LM_DBG("%s is an ipv6 addinfos\n", str_addr);
+	} else {
+		LM_DBG("%s is an unknown addinfos format AF=%d\n",str_addr, info->ai_family);
+		return -1;
+	}
+
+	ret = info->ai_family;
+
+	freeaddrinfo(info);
+
+	return ret;
+}
+
+
+static int get_ip_scope(char *str_addr)
+{
+	struct ifaddrs *ifaddr, *ifa;
+	struct sockaddr_in6 *in6;
+	char str_if_ip[NI_MAXHOST];
+	int ret = -1;
+
+	if (getifaddrs(&ifaddr) == -1) {
+		LM_ERR("getifaddrs() failed: %s\n", gai_strerror(ret));
+		return -1;
+	}
+
+	for (ifa = ifaddr; ifa != NULL; ifa = ifa->ifa_next) {
+		in6 = (struct sockaddr_in6 *)ifa->ifa_addr;
+
+		if (ifa->ifa_addr == NULL)
+			continue;
+
+		if (ifa->ifa_addr->sa_family != AF_INET6)
+			continue;
+
+		ret = getnameinfo(ifa->ifa_addr, sizeof(struct sockaddr_in6),
+		str_if_ip, NI_MAXHOST, NULL, 0, NI_NUMERICHOST);
+		if (ret != 0) {
+			LM_ERR("getnameinfo() failed: %s\n", gai_strerror(ret));
+			return -1;
+		}
+
+		if (strstr(str_if_ip, str_addr)) {
+			LM_INFO("dev: %-8s address: <%s> scope %d\n",
+			ifa->ifa_name, str_if_ip, in6->sin6_scope_id);
+			ret = in6->sin6_scope_id;
+			break;
+		}
+	}
+
+	freeifaddrs(ifaddr);
+
+	return ret;
+}
+
+
+static int bind_force_send_ip(int sock_idx)
+{
+	struct sockaddr_in tmp, ip4addr;
+	struct sockaddr_in6 tmp6, ip6addr;
+	char str_addr[INET_ADDRSTRLEN];
+	char str_addr6[INET6_ADDRSTRLEN];
+	socklen_t sock_len = sizeof(struct sockaddr);
+	int ret, scope;
+
+	switch (force_send_ip_af) {
+		case AF_INET:
+			memset(&ip4addr, 0, sizeof(ip4addr));
+			ip4addr.sin_family = AF_INET;
+			ip4addr.sin_port = htons(0);
+			inet_pton(AF_INET, force_send_ip_str, &ip4addr.sin_addr);
+
+			if (bind(rtpp_socks[sock_idx], (struct sockaddr*)&ip4addr, sizeof(ip4addr)) < 0) {
+				LM_ERR("can't bind socket to required ipv4 interface\n");
+				return -1;
+			}
+
+			memset(&tmp, 0, sizeof(tmp));
+			getsockname(rtpp_socks[sock_idx], (struct sockaddr *) &tmp, &sock_len);
+			inet_ntop(AF_INET, &tmp.sin_addr, str_addr, INET_ADDRSTRLEN);
+			LM_INFO("Binding on %s:%d\n", str_addr, ntohs(tmp.sin_port));
+
+			break;
+
+		case AF_INET6:
+			if ((scope = get_ip_scope(force_send_ip_str)) < 0) {
+				LM_ERR("can't get the ipv6 interface scope\n");
+				return -1;
+			}
+			memset(&ip6addr, 0, sizeof(ip6addr));
+			ip6addr.sin6_family = AF_INET6;
+			ip6addr.sin6_port = htons(0);
+			ip6addr.sin6_scope_id = scope;
+			inet_pton(AF_INET6, force_send_ip_str, &ip6addr.sin6_addr);
+
+			if ((ret = bind(rtpp_socks[sock_idx], (struct sockaddr*)&ip6addr, sizeof(ip6addr))) < 0) {
+				LM_ERR("can't bind socket to required ipv6 interface\n");
+				LM_ERR("ret=%d errno=%d\n", ret, errno);
+				return -1;
+			}
+
+			memset(&tmp6, 0, sizeof(tmp6));
+			getsockname(rtpp_socks[sock_idx], (struct sockaddr *) &tmp6, &sock_len);
+			inet_ntop(AF_INET6, &tmp6.sin6_addr, str_addr6, INET6_ADDRSTRLEN);
+			LM_INFO("Binding on ipv6 %s:%d\n", str_addr6, ntohs(tmp6.sin6_port));
+
+			break;
+
+		default:
+			LM_DBG("force_send_ip_str not specified in .cfg file!\n");
+			break;
+	}
+
+	return 0;
+}
+
 
 static inline int str_eq(const str *p, const char *q) {
 	int l = strlen(q);
@@ -629,7 +641,6 @@ int add_rtpengine_socks(struct rtpp_set * rtpp_list, char * rtpproxy)
 		pnode->rn_url.s[p2 - p1] 	= 0;
 		pnode->rn_url.len 			= p2-p1;
 
-		LM_DBG("url is %s, len is %i\n", pnode->rn_url.s, pnode->rn_url.len);
 		/* Leave only address in rn_address */
 		pnode->rn_address = pnode->rn_url.s;
 		if (strncasecmp(pnode->rn_address, "udp:", 4) == 0) {
@@ -894,8 +905,19 @@ static struct mi_root* mi_show_rtpproxies(struct mi_root* cmd_tree,
 
 			add_rtpp_node_int_info(crt_node, MI_INDEX, MI_INDEX_LEN,
 				crt_rtpp->idx, child, len,string,error);
-			add_rtpp_node_int_info(crt_node, MI_DISABLED, MI_DISABLED_LEN,
-				crt_rtpp->rn_disabled, child, len,string,error);
+			
+			if (( 1 == crt_rtpp->rn_disabled ) && ( crt_rtpp->rn_recheck_ticks == MI_MAX_RECHECK_TICKS)) {
+				if( !(child = add_mi_node_child(crt_node, MI_DUP_VALUE, MI_DISABLED, MI_DISABLED_LEN,
+								MI_DISABLED_PERMANENT, MI_DISABLED_PERMANENT_LEN))) {
+					LM_ERR("cannot add disabled (permanent) message\n");
+					goto error;
+				}
+			}
+			else {
+				add_rtpp_node_int_info(crt_node, MI_DISABLED, MI_DISABLED_LEN,
+					crt_rtpp->rn_disabled, child, len,string,error);
+			}
+
 			add_rtpp_node_int_info(crt_node, MI_WEIGHT, MI_WEIGHT_LEN,
 				crt_rtpp->rn_weight,  child, len, string,error);
 			add_rtpp_node_int_info(crt_node, MI_RECHECK_TICKS,MI_RECHECK_T_LEN,
@@ -954,6 +976,18 @@ mod_init(void)
 		}
 	}
 
+	if (rtp_inst_pv_param.s) {
+		rtp_inst_pv_param.len = strlen(rtp_inst_pv_param.s);
+		rtp_inst_pvar = pv_cache_get(&rtp_inst_pv_param);
+		if ((rtp_inst_pvar == NULL) ||
+			((rtp_inst_pvar->type != PVT_AVP) &&
+	     	 (rtp_inst_pvar->type != PVT_XAVP) &&
+			 (rtp_inst_pvar->type != PVT_SCRIPTVAR))) {
+		LM_ERR("Invalid pvar name <%.*s>\n", rtp_inst_pv_param.len, rtp_inst_pv_param.s);
+		return -1;
+		}
+	}
+
 	if (extra_id_pv_param.s && *extra_id_pv_param.s) {
 		extra_id_pv_param.len = strlen(extra_id_pv_param.s);
 		if(pv_parse_format(&extra_id_pv_param, &extra_id_pv) < 0) {
@@ -980,9 +1014,24 @@ mod_init(void)
 	    setid_avp_type = avp_flags;
 	}
 
+	if (write_sdp_pvar_str.len > 0) {
+		write_sdp_pvar = pv_cache_get(&write_sdp_pvar_str);
+		if (write_sdp_pvar == NULL
+		    || (write_sdp_pvar->type != PVT_AVP &&  write_sdp_pvar->type != PVT_SCRIPTVAR) ) {
+			LM_ERR("write_sdp_pv: not a valid AVP or VAR definition <%.*s>\n",
+		       write_sdp_pvar_str.len, write_sdp_pvar_str.s);
+			return -1;
+		}
+	}
+
 	if (rtpp_strings)
 		pkg_free(rtpp_strings);
 
+	if ((queried_nodes_limit < 1) || (queried_nodes_limit > MAX_RTPP_TRIED_NODES)) {
+		LM_ERR("queried_nodes_limit must be a number in the range 1..50 \n");
+		return -1;
+	}
+
 	if (load_tm_api( &tmb ) < 0)
 	{
 		LM_DBG("could not load the TM-functions - answer-offer model"
@@ -990,6 +1039,14 @@ mod_init(void)
 		memset(&tmb, 0, sizeof(struct tm_binds));
 	}
 
+	/* Determine IP addr type (IPv4 or IPv6 allowed) */
+	force_send_ip_af = get_ip_type(force_send_ip_str);
+	if (force_send_ip_af != AF_INET && force_send_ip_af != AF_INET6 &&
+			strlen(force_send_ip_str) > 0) {
+		LM_ERR("%s is an unknown address\n", force_send_ip_str);
+		return -1;
+	}
+
 	return 0;
 }
 
@@ -1058,19 +1115,28 @@ child_init(int rank)
 
 			rtpp_socks[pnode->idx] = socket((pnode->rn_umode == 6)
 			    ? AF_INET6 : AF_INET, SOCK_DGRAM, 0);
-			if ( rtpp_socks[pnode->idx] == -1) {
+			if (rtpp_socks[pnode->idx] == -1) {
 				LM_ERR("can't create socket\n");
 				freeaddrinfo(res);
 				return -1;
 			}
 
-			if (connect( rtpp_socks[pnode->idx], res->ai_addr, res->ai_addrlen) == -1) {
-				LM_ERR("can't connect to a RTP proxy\n");
-				close( rtpp_socks[pnode->idx] );
+			if (bind_force_send_ip(pnode->idx) == -1) {
+				LM_ERR("can't bind socket\n");
+				close(rtpp_socks[pnode->idx]);
 				rtpp_socks[pnode->idx] = -1;
 				freeaddrinfo(res);
 				return -1;
 			}
+
+			if (connect(rtpp_socks[pnode->idx], res->ai_addr, res->ai_addrlen) == -1) {
+				LM_ERR("can't connect to a RTP proxy\n");
+				close(rtpp_socks[pnode->idx]);
+				rtpp_socks[pnode->idx] = -1;
+				freeaddrinfo(res);
+				return -1;
+			}
+
 			freeaddrinfo(res);
 rptest:
 			pnode->rn_disabled = rtpp_test(pnode, 0, 1);
@@ -1321,7 +1387,7 @@ static bencode_item_t *rtpp_function_call(bencode_buffer_t *bencbuf, struct sip_
 	struct ng_flags_parse ng_flags;
 	bencode_item_t *item, *resp;
 	str callid, from_tag, to_tag, body, viabranch, error;
-	int ret;
+	int ret, queried_nodes;
 	struct rtpp_node *node;
 	char *cp;
 
@@ -1436,17 +1502,22 @@ static bencode_item_t *rtpp_function_call(bencode_buffer_t *bencbuf, struct sip_
 	if(msg->id != current_msg_id)
 		active_rtpp_set = default_rtpp_set;
 
+	queried_nodes = 0;
 	do {
+		if (++queried_nodes > queried_nodes_limit) {
+			LM_ERR("queried nodes limit reached\n");
+			goto error;
+		}
 		node = select_rtpp_node(callid, 1);
 		if (!node) {
 			LM_ERR("no available proxies\n");
 			goto error;
 		}
-
 		cp = send_rtpp_command(node, ng_flags.dict, &ret);
 	} while (cp == NULL);
 	LM_DBG("proxy reply: %.*s\n", ret, cp);
 
+	set_rtp_inst_pvar(msg, &node->rn_url);
 	/*** process reply ***/
 
 	resp = bencode_decode_expect(bencbuf, cp, ret, BENCODE_DICTIONARY);
@@ -1630,7 +1701,7 @@ send_rtpp_command(struct rtpp_node *node, bencode_item_t *dict, int *outlen)
 				LM_ERR("can't send command to a RTP proxy\n");
 				goto badproxy;
 			}
-			while ((poll(fds, 1, rtpengine_tout * 1000) == 1) &&
+			while ((poll(fds, 1, rtpengine_tout_ms) == 1) &&
 			    (fds[0].revents & POLLIN) != 0) {
 				do {
 					len = recv(rtpp_socks[node->idx], buf, sizeof(buf)-1, 0);
@@ -2062,6 +2133,7 @@ rtpengine_offer_answer(struct sip_msg *msg, const char *flags, int op, int more)
 	bencode_item_t *dict;
 	str body, newbody;
 	struct lump *anchor;
+	pv_value_t pv_val;
 
 	dict = rtpp_function_call_ok(&bencbuf, msg, op, flags, &body);
 	if (!dict)
@@ -2078,14 +2150,28 @@ rtpengine_offer_answer(struct sip_msg *msg, const char *flags, int op, int more)
 	if (more)
 		body_intermediate = newbody;
 	else {
-		anchor = del_lump(msg, body.s - msg->buf, body.len, 0);
-		if (!anchor) {
-			LM_ERR("del_lump failed\n");
-			goto error_free;
-		}
-		if (!insert_new_lump_after(anchor, newbody.s, newbody.len, 0)) {
-			LM_ERR("insert_new_lump_after failed\n");
-			goto error_free;
+		if (write_sdp_pvar!= NULL) {
+			pv_val.rs = newbody;
+			pv_val.flags = PV_VAL_STR;
+
+			if (write_sdp_pvar->setf(msg,&write_sdp_pvar->pvp, (int)EQ_T, &pv_val) < 0)
+			{
+				LM_ERR("error setting pvar <%.*s>\n", write_sdp_pvar_str.len, write_sdp_pvar_str.s);
+				goto error_free;
+			}
+
+			pkg_free(newbody.s);
+
+		} else {
+			anchor = del_lump(msg, body.s - msg->buf, body.len, 0);
+			if (!anchor) {
+				LM_ERR("del_lump failed\n");
+				goto error_free;
+			}
+			if (!insert_new_lump_after(anchor, newbody.s, newbody.len, 0)) {
+				LM_ERR("insert_new_lump_after failed\n");
+				goto error_free;
+			}
 		}
 	}
 
@@ -2110,41 +2196,40 @@ start_recording_f(struct sip_msg* msg, char *foo, char *bar)
 	return rtpengine_rtpp_set_wrap(msg, rtpengine_start_recording_wrap, NULL, 1);
 }
 
-/*
- * Returns the current RTP-Statistics from the RTP-Proxy
- */
-static int
-pv_get_rtpstat_f(struct sip_msg *msg, pv_param_t *param,
-		  pv_value_t *res)
-{
+static int rtpengine_rtpstat_wrap(struct sip_msg *msg, void *d, int more) {
+	void **parms;
+	pv_param_t *param;
+	pv_value_t *res;
 	bencode_buffer_t bencbuf;
-	bencode_item_t *dict, *tot, *in, *out;
+	bencode_item_t *dict, *tot, *rtp, *rtcp;
 	static char buf[256];
 	str ret;
 
+	parms = d;
+	param = parms[0];
+	res = parms[1];
+
 	dict = rtpp_function_call_ok(&bencbuf, msg, OP_QUERY, NULL, NULL);
 	if (!dict)
 		return -1;
 
 	tot = bencode_dictionary_get_expect(dict, "totals", BENCODE_DICTIONARY);
-	in = bencode_dictionary_get_expect(tot, "input", BENCODE_DICTIONARY);
-	in = bencode_dictionary_get_expect(in, "rtp", BENCODE_DICTIONARY);
-	out = bencode_dictionary_get_expect(tot, "output", BENCODE_DICTIONARY);
-	out = bencode_dictionary_get_expect(out, "rtp", BENCODE_DICTIONARY);
+	rtp = bencode_dictionary_get_expect(tot, "RTP", BENCODE_DICTIONARY);
+	rtcp = bencode_dictionary_get_expect(tot, "RTCP", BENCODE_DICTIONARY);
 
-	if (!in || !out)
+	if (!rtp || !rtcp)
 		goto error;
 
 	ret.s = buf;
 	ret.len = snprintf(buf, sizeof(buf),
-			"Input: %lli bytes, %lli packets, %lli errors; "
-			"Output: %lli bytes, %lli packets, %lli errors",
-			bencode_dictionary_get_integer(in, "bytes", -1),
-			bencode_dictionary_get_integer(in, "packets", -1),
-			bencode_dictionary_get_integer(in, "errors", -1),
-			bencode_dictionary_get_integer(out, "bytes", -1),
-			bencode_dictionary_get_integer(out, "packets", -1),
-			bencode_dictionary_get_integer(out, "errors", -1));
+			"RTP: %lli bytes, %lli packets, %lli errors; "
+			"RTCP: %lli bytes, %lli packets, %lli errors",
+			bencode_dictionary_get_integer(rtp, "bytes", -1),
+			bencode_dictionary_get_integer(rtp, "packets", -1),
+			bencode_dictionary_get_integer(rtp, "errors", -1),
+			bencode_dictionary_get_integer(rtcp, "bytes", -1),
+			bencode_dictionary_get_integer(rtcp, "packets", -1),
+			bencode_dictionary_get_integer(rtcp, "errors", -1));
 
 	bencode_buffer_free(&bencbuf);
 	return pv_get_strval(msg, param, res, &ret);
@@ -2154,3 +2239,35 @@ error:
 	return -1;
 }
 
+/*
+ * Returns the current RTP-Statistics from the RTP-Proxy
+ */
+static int
+pv_get_rtpstat_f(struct sip_msg *msg, pv_param_t *param,
+		  pv_value_t *res)
+{
+	void *parms[2];
+
+	parms[0] = param;
+	parms[1] = res;
+
+	return rtpengine_rtpp_set_wrap(msg, rtpengine_rtpstat_wrap, parms, 1);
+}
+
+int set_rtp_inst_pvar(struct sip_msg *msg, const str * const uri) {
+	pv_value_t val;
+
+	if (rtp_inst_pvar == NULL)
+		return 0;
+
+	memset(&val, 0, sizeof(pv_value_t));
+	val.flags = PV_VAL_STR;
+	val.rs = *uri;
+
+	if (rtp_inst_pvar->setf(msg, &rtp_inst_pvar->pvp, (int)EQ_T, &val) < 0)
+	{
+		LM_ERR("Failed to add RTP Engine URI to pvar\n");
+		return -1;
+	}
+	return 0;
+}
